#include "settings.h"
#include "ui_settings.h"

#include <QDebug>
#include <QMessageBox>
#include <QProcess>

Settings::Settings(QWidget* parent)
    : QDialog(parent)
    , ui(new Ui::Settings)
{
    ui->setupUi(this);
    QScopedPointer<QProcess> proc(new QProcess());
    proc->start("snapper", QStringList() << "get-config");
    if (!proc->waitForFinished()) {
        errorMessage("Failed to load settings.");
        throw std::exception();
    }
    if (proc->exitCode() != 0) {
        errorMessage("Failed to load settings.");
        throw std::exception();
    }
    unsigned int line_num = 0;
    while (proc->canReadLine()) {
        auto line = QString::fromUtf8(proc->readLine());
        if (line_num < 2) {
            line_num++;
            continue;
        }
        auto parts = line.split("|");
        settingsmap[parts[0].simplified()] = parts[1].simplified();
    }
    ui->checkBox_timeline->setChecked(settingsmap.value("TIMELINE_CREATE", "false") == "yes");
    ui->hourly->setValue(settingsmap.value("TIMELINE_LIMIT_HOURLY", "5").toUInt());
    ui->daily->setValue(settingsmap.value("TIMELINE_LIMIT_DAILY", "7").toUInt());
    ui->weekly->setValue(settingsmap.value("TIMELINE_LIMIT_WEEKLY", "0").toUInt());
    ui->monthly->setValue(settingsmap.value("TIMELINE_LIMIT_MONTHLY", "0").toUInt());
    ui->yearly->setValue(settingsmap.value("TIMELINE_LIMIT_YEARLY", "0").toUInt());
    ui->pacman->setValue(settingsmap.value("NUMBER_LIMIT", "10").toUInt());
}

Settings::~Settings()
{
    delete ui;
}

void Settings::errorMessage(QString message)
{
    QMessageBox::critical(this, "An error has occoured", message);
}

void Settings::on_buttonBox_accepted()
{
    QMap<QString, QString> new_settings;
    new_settings["TIMELINE_CREATE"] = ui->checkBox_timeline->isChecked() ? "yes" : "no";
    new_settings["TIMELINE_LIMIT_HOURLY"] = QString::number(ui->hourly->value());
    new_settings["TIMELINE_LIMIT_DAILY"] = QString::number(ui->daily->value());
    new_settings["TIMELINE_LIMIT_WEEKLY"] = QString::number(ui->weekly->value());
    new_settings["TIMELINE_LIMIT_MONTHLY"] = QString::number(ui->monthly->value());
    new_settings["TIMELINE_LIMIT_YEARLY"] = QString::number(ui->yearly->value());
    new_settings["NUMBER_LIMIT"] = QString::number(ui->pacman->value());

    QScopedPointer<QProcess> proc(new QProcess());
    QStringList params;
    params << "set-config";
    for (const auto& setting : new_settings.keys()) {
        params << setting + "=" + new_settings[setting];
    }
    proc->start("snapper", params);
    if (!proc->waitForFinished())
        errorMessage("Failed to save settings.");
    else if (proc->exitCode() != 0)
        throw std::runtime_error("Failed to save settings.");
}
